/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.api;

import cf.common.Engine;
import cf.common.Event;
import cf.util.Log;

public class ApplicationEvent extends Event
{

  private double creationTime; // Creation time of this information.
  private double activationTime; // Activation time of this information.
  private double expirationTime; // Expiration time of this information.

  public CyberNode node;

  /** Observers. */

  public String creator() { return app.name(); }
  public double creationTime() { return creationTime; }
  public double activationTime() { return activationTime; }
  public double expirationTime() { return expirationTime; }
  public boolean expired() { return Engine.time() >= expirationTime; }

  private CyberApplication app;

  public ApplicationEvent(CyberApplication app, double activationDelay)
  {
    super(activationDelay);

    if(app!=null)
    {
    this.app = app;
    this.creationTime = app.time();
    }
    this.activationTime = this.creationTime + activationDelay;
    this.expirationTime = Double.POSITIVE_INFINITY;
  }

  public ApplicationEvent(CyberApplication app, double activationDelay, double expirationDelay)
  {
    super(activationDelay);

    assert(activationDelay < expirationDelay);

    if(app!=null)
    {
    this.app = app;
    this.creationTime = app.time();
    }
    this.activationTime = this.creationTime + activationDelay;
    this.expirationTime = this.creationTime + expirationDelay;
  }

  public boolean related(ApplicationEvent e)
  {
    return false;
  }

  public boolean subsumes(ApplicationEvent e)
  {
    return false;
  }

  public boolean replaces(ApplicationEvent e)
  {
    return false;
  }

  public CyberApplication application()
  {
    return app;
  }

  public void handle()
  {
    app.handleEvent(this);
  }

  public String logRepShort()
  {
    String r = "";
    if(app != null)
      r = Log.getClassName(this) + "(" + app.name() + "," + time() + ")";
    else
      r = Log.getClassName(this) +  "( " + time()+" )";
    return r;
  }

  public String logRepLong()
  {
    return logRepShort();
  }
}
