/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.api;

import cf.common.KnowledgeBase;
import cf.common.Param;
import cf.util.Log;

public class CyberApp implements CyberApplication
{
    public CyberNode node;

    public String name()
    {
        return node.name();
    }

    public double time()
    {
        return node.time();
    }

    public String parentname()
    {
        return node.parentname();
    }

    public CyberApp(CyberNode node)
    {
        this.node = node;
        addApplication(this);
    }

    public void initialize()
    {
    }

    public void addApplication(CyberApplication app)
    {
        node.addApplication(app);
    }

    public void handleKnowledgeExpired(ApplicationKnowledge k) { }

    public void postEvent(ApplicationEvent e)
    {
        node.postEvent(e);
    }

    public void handleEvent(ApplicationEvent e)
    {
        handleEvent();
    }
  
    public KnowledgeBase kb()
    {
  	return node.kb();
    }

    public void postKnowledge(ApplicationKnowledge k)
    {
        node.postKnowledge(k);
    }

    public void handleKnowledge(ApplicationKnowledge k)
    {
    }

    public void postEvent(double delay)
    {
        node.postEvent(new ApplicationEvent(this,delay));
    }

    public void handleEvent()
    {
    }

    public String logRepLong()
    {
        return "";
    }

    public String logRepShort()
    {
        //if(Param.SILENT){ return ""; }
        return Log.getClassName(this) + "(" + name() + ")";
    }
    
    public void log(String msg)
    {
        //if(Param.SILENT){ return; }
        node.log(logRepShort() + " > " + msg);
    }
    
    public void log(int level, String msg)
    {
        //if(Param.SILENT){ return; }
        node.log(level, logRepShort() + " > " + msg);
    }
    
    //PADO
    public void terminate()
    {
        node.terminate();
    }
}
