/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.approximation;


import java.io.IOException;

import cf.api.CyberApplication;
import cf.common.Model;
import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.sim.SimWorld;

class ApproxSimulation extends SimEngine
{
    public boolean invariantProperty(SimWorld world)
    {
	int critical = 0;

	for(Model m : world.allModels())
	    {
		if(m instanceof SimNode)
		    {
			SimNode sn = (SimNode)m;
			for(CyberApplication app : sn.allApplications())
			    {
				ApproxApp mn = (ApproxApp)app;
				if(mn.critical) critical++;
			    }
		    }
	    }
	
	return critical <= 1;
    }

    public SimWorld createSimWorld()
    {
	return new ApproxWorld();
    }
}

class ApproxMain
{
    public static void main (String args[]) throws IOException
    {
	ApproxSimulation sim = new ApproxSimulation();
	sim.run();

	int ip=0, tp=1; 
	
	// generic approximation (epsilon, delta) 
	sim.generic_approx_run(0.1, 0.05);

    // optimal approximation (epsilon, delta, c, property)
	sim.optimal_approx_run(0.1, 0.05, 2.0, ip); 
	sim.optimal_approx_run(0.1, 0.05, 2.0, tp);  

  	// blackbox testing (thresthold, rounds)
	sim.blackbox_approx_run(0.8, 100);

	// sequential testing 
	// (type_1_error, type_2_error, threshold, indifference_region, property)
	sim.seqtest_approx_run(0.1, 0.1, 0.8, 0.05, ip); 
	sim.seqtest_approx_run(0.1, 0.1, 0.1, 0.05, tp); 

	// quantitative analysis 
	// (error_bound, confidence_interval, jbtest_pval,
	// min_rounds, max_rounds, sample_increase, property)
	sim.qa_approx_run(0.05, 0.01, 0.05, 100, 1000, 0.1, ip); 
	sim.qa_approx_run(0.1, 0.05, 0.05, 100, 1000, 0.1, tp);  
    }
}
