/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.consensus; 


import java.io.IOException;

import cf.api.CyberApplication;
import cf.common.Model;
import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.sim.SimWorld;

class ConsensusSimulation extends SimEngine
{
    public boolean terminationProperty(SimWorld world)
    {
	for(Model n : world.allModels()) 
	    {
		if(n instanceof SimNode)
		    {
			for(CyberApplication an : n.allApplications())
			    {
				if(an instanceof ConsensusApp)
				    {
					ConsensusApp can = (ConsensusApp)an;
					if(!(can.myproposal >= 0)) return false;
				    }
			    }
		    }
	    }

	for(Model n : world.allModels()) 
	    {
		if(n instanceof SimNode)
		    {
			for(Model m : world.allModels()) 
			    {
				if(m instanceof SimNode)
				    {
					for(CyberApplication an : n.allApplications())
					    {
						if(an instanceof ConsensusApp)
						    {
							for(CyberApplication am : m.allApplications())
							    {
								if(am instanceof ConsensusApp)
								    {
									ConsensusApp can = (ConsensusApp)an;
									ConsensusApp cam = (ConsensusApp)am;
									if(!(can.myproposal == cam.myproposal)) return false;
								    }
							    }
						    }
					    }
				    }
			    }
		    }
	    }
	return true;
    }

    public SimWorld createSimWorld()
    {
	return new ConsensusWorld();
    }
}

class ConsensusMain
{
    public static void main (String args[]) throws IOException
    {
	ConsensusSimulation sim = new ConsensusSimulation();
	sim.run();
    }
}
