/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.cyberping;

import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.text.DecimalFormat;
import java.util.LinkedList;

import cf.api.ApplicationKnowledge;
import cf.api.CyberApp;
import cf.api.CyberNode;
import cf.real.PingKnowledge;
import cf.real.PingResponseKnowledge;
import cf.util.Log;

class CyberPingApp extends CyberApp
{
  public CyberPingApp(CyberNode node, double waitTime)
  {
    super(node);
    this.waitTime = waitTime;
    current_seq = -1;
    transmitted = 0;
    received = 0;
    Log.stdout.println("wait time: " + this.waitTime);
  }

  int current_seq;
  double waitTime;
  double sent_time;
  long myEngineId;
  int transmitted;
  int received;
  LinkedList<Double> rtt = new LinkedList<Double>();

  public void initialize()
  {
    postEvent(waitTime);
  }

  void sendPing()
  {
	Log.stdout.println("posting ping knowledge - sequence number: " + (current_seq+1));
    PingKnowledge pk = new PingKnowledge(this, current_seq+1);
    sent_time = pk.creationTime();
    postKnowledge(pk);
    myEngineId = pk.engineId();
    transmitted++;
  }

  public void handleEvent()
  {
    sendPing();
    current_seq++;
    postEvent(waitTime);
  }

  public void handleKnowledge(ApplicationKnowledge k)
  {
    if(k instanceof PingResponseKnowledge) 
    {
      PingResponseKnowledge prk = (PingResponseKnowledge)k;
      //System.out.println("mERONG " + seq + " " + current_seq + " " + prk.engineId() + " " + myEngineId);

      if( prk.seq() == current_seq && prk.engineId()!= myEngineId )
      {
        try 
        {
          double t = prk.creationTime() - sent_time;
          double rtt = time() - sent_time;
          received++;

          ByteArrayOutputStream b_out = new ByteArrayOutputStream();
          ObjectOutputStream o_out = new ObjectOutputStream(b_out);
          o_out.writeObject(prk);
          o_out.close();

          int size = b_out.toByteArray().length;

          DecimalFormat format = new DecimalFormat("#.#####");

          String time = format.format(t);
          String rtttime = format.format(rtt);

          log("received ping response from " + prk.creator() + " sequence number: " + prk.seq() + " round-trip time: " + rtttime + "s");
          Log.stdout.println("received ping response from " + prk.creator() + " sequence number: " + prk.seq() + " round-trip time: " + rtttime + "s");
        }
        catch(Exception e)
        {
          Log.stderr.println("CyberPing.handleKnowledge - " + e);
        }
      }
    }
  }

  public void print()
  {
	Log.stdout.println("total transmitted: " + transmitted);
	Log.stdout.println("total received: " + received);
  }
}
