/*

  Copyright 2010-2013 SRI International  

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.mason.scattering2d; 

import java.io.IOException;

import cf.api.CyberApplication;
import cf.common.Node;
import cf.sim.SimEngine;
import cf.sim.mason.SimWorldMason;


class ScatteringSimulation2D extends SimEngine
{

	double distance(ScatteringApp2D san, ScatteringApp2D sam)
	{
		double dx = san.posx() - sam.posx();
		double dy = san.posy() - sam.posy();
		return Math.sqrt(dx*dx+dy*dy);
	}

	public int countNeighbors(SimWorldMason world, ScatteringApp2D san)
	{
		int c = 0;
		for(Node m : world.allNodes()) 
		{
			for(CyberApplication am : m.allApplications())
			{
				ScatteringApp2D sam = (ScatteringApp2D)am;
				if(san != sam)
				{
					if(distance(san,sam) < 10) c++;
				}
			}
		}	
		return c;
	}

	public boolean invariantProperty(SimWorldMason world)
	{
		for(Node n : world.allNodes()) 
		{
			for(CyberApplication an : n.allApplications())
			{
				ScatteringApp2D san = (ScatteringApp2D)an;
				if(countNeighbors(world,san) == 0) return false;
			}
		}

		return true;
	}

	public SimWorldMason createSimWorld()
	{
		return new ScatteringWorld2D();
	}
}

class ScatteringMain2D
{
	public static void main (String args[]) throws IOException
	{
		ScatteringSimulation2D sim = new ScatteringSimulation2D();
		sim.run();
	}
}
