/*

  Copyright 2010-2013 SRI International  

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.mason.scattering3d; 

import java.io.IOException;

import cf.api.CyberApplication;
import cf.common.Model;
import cf.common.Node;
import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.sim.mason.GUIState2DMason;
import cf.sim.mason.SimWorld2DMason;
import cf.sim.mason.SimWorldMason;

import sim.engine.SimState;


class ScatteringSimulation3D extends SimEngine
{

	double distance(ScatteringApp3D san, ScatteringApp3D sam)
	{
		double dx = san.posx() - sam.posx();
		double dy = san.posy() - sam.posy();
		double dz = san.posy() - sam.posz();
		return Math.sqrt(dx*dx+dy*dy+dz*dz);
	}

	public int countNeighbors(SimWorldMason world, ScatteringApp3D san)
	{
		int c = 0;
		for(Node m : world.allNodes()) 
		{
				for(CyberApplication am : m.allApplications())
				{
					ScatteringApp3D sam = (ScatteringApp3D)am;
					if(san != sam)
					{
						if(distance(san,sam) < 10) c++;
					}
				}
		}	
		return c;
	}

	public boolean invariantProperty(SimWorldMason world)
	{
		for(Node n : world.allNodes()) 
		{
			if(n instanceof SimNode)
			{
				for(CyberApplication an : n.allApplications())
				{
					ScatteringApp3D san = (ScatteringApp3D)an;
					if(countNeighbors(world,san) == 0) return false;
				}
			}
		}

		return true;
	}

	public SimWorldMason createSimWorld()
	{
		return new ScatteringWorld3D();
	}
}

class ScatteringMain3D
{
	public static void main (String args[]) throws IOException
	{
		ScatteringSimulation3D sim = new ScatteringSimulation3D();
		sim.run();
	}
}
