/*

  Copyright 2010-2013 SRI International  

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.mason.simulatedannealing; 

import cf.sim.pn.Place;
import cf.sim.pn.SimMobility;
import cf.sim.SimNode;
import cf.sim.dtn.LinkType;
import cf.sim.dtn.SimLink;
import cf.sim.mason.SimPosition2DMason;
import cf.sim.mason.SimWorld2DMason;
import cf.sim.mason.devices.SimPosition2D;

public class SimulatedAnnealingWorld extends SimWorld2DMason
{    
    public static final int NUM_ROBOTS = 10;

    SimNode robot[] = new SimNode[NUM_ROBOTS];
    SimPosition2D pos[] = new SimPosition2D[NUM_ROBOTS];
    SimulatedAnnealingApp app[] = new SimulatedAnnealingApp[NUM_ROBOTS];
    
    public SimulatedAnnealingWorld()
    {		
    	super();
    	
        for(int i = 0; i < NUM_ROBOTS; i++)
    	{
    		robot[i] = new SimNode(this,"RobotNode"+i); 

    		pos[i] = new SimPosition2DMason(this,robot[i],"PosDev"+i);
    		//pos[i].setPos(SimWorld2DMason.XMAX*0.2+i*(Math.random()-0.5),SimWorld2DMason.YMAX*0.2+i*(Math.random()-0.5));
    		pos[i].setPos(SimWorld2DMason.XMAX*0.2,SimWorld2DMason.YMAX*0.2+i*10);
    		
    		app[i] = new SimulatedAnnealingApp(robot[i],pos[i]);
    	}
        
        // fully connected sim model

		double cost = 1.0;
		double latency = 0.0;
		double bandwidth = 100000;
		double errorRate = 0;
		
        for(int i = 0; i < NUM_ROBOTS; i++)
        {
        	for(int j = i+1; j < NUM_ROBOTS; j++)
        	{
        		SimLink l = addSymmetricLink(robot[i],robot[j],LinkType.Discovered,cost,latency,bandwidth,errorRate);
        		l.initLinkState(true);
        	}
        }
    }
}
