/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.pn.sensorrobots;

import cf.sim.SimNode;
import cf.sim.SimWorld;
import cf.sim.basic.devices.SimCamera;
import cf.sim.basic.devices.SimMotionSensor;
import cf.sim.basic.devices.SimNoiseSensor;
import cf.sim.dtn.LinkType;
import cf.sim.dtn.SimLink;
import cf.sim.pn.Place;
import cf.sim.pn.SimMobility;
import cf.sim.pn.Transition;
import cf.sim.pn.devices.SimPositionMobility;

public class SensorWorld extends SimWorld
{
  public SensorWorld()
  {		
    SimNode root = new SimNode(this,"RootNode");
    new RootApp(root);

    SimNode [] robot = new SimNode[1];
    for(int i = 0; i < 1; i++)
    {
      robot[i] = new SimNode(this,"RobotNode"+i); 
      SimNode cam = new SimNode(this, robot[i], "CameraNode"+i);
      SimNode position = new SimNode(this,robot[i],  "PositionNode"+i);
      SimNode motion_sensor = new SimNode(this,robot[i],  "MotionSensorNode"+i);
      SimNode noise_sensor = new SimNode(this, robot[i], "NoiseSensorNode"+i);


      new RobotApp(robot[i]);
      new CameraApp(cam,new SimCamera(this,robot[i],"CameraDev"+i));
      new PositionApp(position,new SimPositionMobility(this,robot[i],"PositionDev"+i));
      new MotionSensorApp(motion_sensor,new SimMotionSensor(this,robot[i],"MotionSensorDev"+i));
      new NoiseSensorApp(noise_sensor,new SimNoiseSensor(this,robot[i],"NoiseSensorDev"+i));
    }

    // link
    SimLink l;
    l = this.addSymmetricLink(robot[0], root, LinkType.Wireless );
    this.handleLinkChange(true, l);
    SimLink r = l.reverse();

    // Mobility environment
    SimMobility rs = new SimMobility(this);

    /// place
    Place PA = rs.addPlace("pa");
    Place PB = rs.addPlace("pb");
    Place PC = rs.addPlace("pc");
    Place PD = rs.addPlace("pd");
    Place PE = rs.addPlace("pe");

    // initial place
    rs.addToken(PA, root);
    rs.addToken(PA, robot[0]);

    // transistion 
    Transition tab = rs.addTransition(PA, "st_ab", PB);
    Transition tbc = rs.addTransition(PB, "st_bc", PC);
    Transition tbe = rs.addTransition(PB, "st_be", PE);
    Transition ted = rs.addTransition(PE, "st_ed", PD);

    Transition tba = rs.addTransition(PB, "st_ba", PA);
    Transition tcb = rs.addTransition(PC, "st_cb", PB);
    Transition teb = rs.addTransition(PE, "st_eb", PB);
    Transition tde = rs.addTransition(PD, "st_de", PE);
    /*

       pa  <-------->  pb <--------> pc
                       |
                       |
                       |
       pd  <-------->  pe

    */


    // link connectivity
    rs.addSymmetricContact(l,PA,PB);
    rs.addSymmetricContact(r,PA,PB);

    rs.addSymmetricContact(l,PB,PC);
    rs.addSymmetricContact(r,PB,PC);

    rs.addSymmetricContact(l,PB,PE);
    rs.addSymmetricContact(r,PB,PE);

    rs.addSymmetricContact(l,PE,PD);
    rs.addSymmetricContact(r,PE,PD);

    rs.addContact(l,PA,PA);
    rs.addContact(r,PA,PA);

    rs.addContact(l,PB,PB);
    rs.addContact(r,PB,PB);

    rs.addContact(l,PC,PC);
    rs.addContact(r,PC,PC);

    rs.addContact(l,PD,PD);
    rs.addContact(r,PD,PD);

    rs.addContact(l,PE,PE);
    rs.addContact(r,PE,PE);
//    rs.start();
    rs.setup();

  }
}
