/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.scattering;

import java.io.IOException;

import cf.api.CyberApplication;
import cf.common.Model;
import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.sim.SimWorld;

class ScatteringSimulation extends SimEngine
{

    double distance(ScatteringApp san, ScatteringApp sam)
    {
	double dx = san.posx() - sam.posx();
	double dy = san.posy() - sam.posy();
	return Math.sqrt(dx*dx+dy*dy);
    }

    public int countNeighbors(SimWorld world, ScatteringApp san)
    {
	int c = 0;
	for(Model m : world.allModels()) 
	    {
		if(m instanceof SimNode)
		    {
			for(CyberApplication am : m.allApplications())
			    {
				ScatteringApp sam = (ScatteringApp)am;
				if(san != sam)
				    {
					if(distance(san,sam) < 10) c++;
				    }
			    }
		    }
	    }	
	return c;
    }

    public boolean invariantProperty(SimWorld world)
    {
	for(Model n : world.allModels()) 
	    {
		if(n instanceof SimNode)
		    {
			for(CyberApplication an : n.allApplications())
			    {
				ScatteringApp san = (ScatteringApp)an;
				if(countNeighbors(world,san) == 0) return false;
			    }
		    }
	    }
	
	return true;
    }

    public SimWorld createSimWorld()
    {
	return new ScatteringWorld();
    }
}

class ScatteringMain
{
    public static void main (String args[]) throws IOException
    {
	ScatteringSimulation sim = new ScatteringSimulation();
	sim.run();
    }
}
