/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.sensors;

import cf.api.ApplicationKnowledge;
import cf.api.CyberApp;
import cf.api.CyberApplication;
import cf.api.CyberNode;
import cf.api.devices.Area;
import cf.api.devices.Camera;
import cf.api.devices.CameraApplication;
import cf.api.devices.Image;
import cf.api.devices.MotionSensor;
import cf.api.devices.MotionSensorApplication;
import cf.api.devices.NoiseSensor;
import cf.api.devices.NoiseSensorApplication;

class MotionKnowledge extends ApplicationKnowledge
{
    Area area;
    
    MotionKnowledge(CyberApplication creator)
    {
	super(creator);
    }
}

class NoiseKnowledge extends ApplicationKnowledge
{
    Area area;
    
    NoiseKnowledge(CyberApplication creator)
    {
	super(creator);
    }
}

class ImageKnowledge extends ApplicationKnowledge
{
    Area area;
    Image image;
    
    ImageKnowledge(CyberApplication creator)
    {
	super(creator);
    }

    public boolean replaces(ApplicationKnowledge k)
    {
	return false; // no replacement
    }

    public String logRepLong()
    {
	return super.logRepLong() + " Image: " + image.logRepLong();
    }
}

class RootApp extends CyberApp
{
    public void handleKnowledge(ApplicationKnowledge knew)
    {
	if(knew instanceof ImageKnowledge) handleImage((ImageKnowledge)knew);
    }

    RootApp(CyberNode node)
    {
	super(node);
    }
    
    void handleImage(ImageKnowledge kimage)
    {
	log("handling image");
    }
}

class MotionSensorApp extends CyberApp implements MotionSensorApplication
{    
    MotionSensor sensor;

    MotionSensorApp(CyberNode node, MotionSensor sensor)
    {
	super(node);
	this.sensor = sensor;
	sensor.addApplication(this);
    }

    public void handleMotion()
    {
	log("handling motion");
	MotionKnowledge kmotion = new MotionKnowledge(this);
	postKnowledge(kmotion);
    }
}

class NoiseSensorApp extends CyberApp implements NoiseSensorApplication
{    
    NoiseSensor sensor;

    NoiseSensorApp(CyberNode node, NoiseSensor sensor)
    {
	super(node);
	this.sensor = sensor;
	sensor.addApplication(this);
    }

    public void handleNoise()
    {
	log("handling noise");
	NoiseKnowledge knoise = new NoiseKnowledge(this);
	postKnowledge(knoise);
    }
}

class CameraApp extends CyberApp implements CameraApplication
{    
    Camera camera;
    Area area;

    CameraApp(CyberNode node, Camera camera)
    {
	super(node);
	this.camera = camera;
	camera.addApplication(this);
    }

    public void handleKnowledge(ApplicationKnowledge knew)
    {
	if(knew instanceof MotionKnowledge) handleMotionKnowledge((MotionKnowledge)knew);
	else if(knew instanceof NoiseKnowledge) handleNoiseKnowledge((NoiseKnowledge)knew);
    }

    void handleMotionKnowledge(MotionKnowledge kmotion)
    {	
	log("handling motion knowledge");

	if(kmotion.area == area)
	    {
		Image image = camera.takeImage(area);
		ImageKnowledge kimage = new ImageKnowledge(this);
		kimage.area = area;
		kimage.image = image;
		postKnowledge(kimage);
	    }
    }

    void handleNoiseKnowledge(NoiseKnowledge knoise)
    {	
	log("handling noise knowledge");

	if(knoise.area == area)
	    {
		Image image = camera.takeImage(area);
		ImageKnowledge kimage = new ImageKnowledge(this);
		kimage.area = area;
		kimage.image = image;
		postKnowledge(kimage);
	    }
    }

}

//Dummy class to keep ant happy
class SensorApp extends CyberApp
{
 public SensorApp(CyberNode node)
 {
	super(node);
 }
}