/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.common;

import java.util.PriorityQueue;

import cf.api.ApplicationEvent;
import cf.common.Node.KnowledgeHandleEvent;
import cf.common.Node.KnowledgeUpdateEvent;
import cf.real.RealNode;
import cf.util.Log;
import cf.real.RealWorld;

public class CyberThread extends Thread
{
  public CyberThread(World world , Model model)
  {
    this.world = world;
    this.model = model;
    if(!Param.SILENT)Engine.log.println(1, "Thread " + model.name() + " created " );
    //System.out.println("Thread " + model.name() + " created " );
  }

  private World world;
  private Model model;
  private Node node;
  int doneId;

  public void run () 
  {
    model.initialize();

    /* if device case, just call initialize and return */
    if((model instanceof Node)) 
      node = (Node)model;
    else
      return;

    node.km().initialize();
    PriorityQueue<Event> q = node.queue();

    int count = 0;

    while(true)
    {
      boolean done =false;
      synchronized(world) {
        if(world.cyberThreadStart) done = true;
      }
      if(done) break;
    }
    
    //PADO
    int node_num = world.allNodes().size();
    //END of PADO
    
    while(true)
    {		
      if(Engine.time() > Param.MaxTime)
      {
        Log.stdout.println("Events processed: " + count + " - maximum time reached" + " (" + node.name() +")");
        break;
      }
      
      //PADO
      if( node instanceof RealNode && ((RealNode)node).isNetworkNode() )
      {
        if(((RealWorld)world).numTerminated() == node_num)
        {
          Log.stdout.println("All node terminated - from network node ");
          break;
        }
      }
      else if( node.explicit_terminated() )
      {
        Log.stdout.println("node" + node.name() + " explictly terminated");
        break;
      }
      //END of PADO

      Event e;
      synchronized(q) {
        e = q.peek();
      }
      if(e == null) 
      {
        //Log.stdout.println("Events processed: " + count + " - no further events"+"\t"+model.name());
        continue;
      }

      node.myThread = Thread.currentThread();
      int maxsleep = (int)(1000.0 * (e.time() - Engine.time()));
      if(maxsleep > 0)
	  {
	      if(maxsleep > Param.MaxSleepTime) maxsleep = Param.MaxSleepTime;
	      try { Thread.sleep(maxsleep); } catch(InterruptedException ex) {}
	  }
      
      while( Engine.time() >= e.time() )
      {
    	  //PADO
    	  if( node.explicit_terminated() )
          {
            if( node instanceof RealNode && ((RealNode)node).isNetworkNode() )
            {
              if(((RealWorld)world).numTerminated() == node_num)
              {
                Log.stdout.println("All node terminated - from network node ");
                break;
              }

              /* ignore remain application events */
              if( e instanceof ApplicationEvent )
              {
                if( !(e instanceof KnowledgeHandleEvent) && !(e instanceof KnowledgeUpdateEvent))
                {
                  synchronized(q) {
                    q.remove(e);
                  }
                  break;
                }
              }
            }
            else
            {
              Log.stdout.println("node" + node.name() + " explictly terminated");
              break;
            }
          }
    	  //END of PADO
    	  
        if(!e.cancelled()) 
        {
          if(!Param.SILENT) Engine.log.println(1, "--- Real Time: " + Log.tf.format(Engine.time()) + " --- Event: " + e.logRepShort() + " (" + node.name() +")");
          e.trigger();
        }
        count++;
        synchronized(q) {
          q.remove(e);
          e = q.peek();
        }
        if(e == null) 
          break;
      }
    }
  }
}
