/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.common;

import cf.util.Log;
import sim.engine.SimState;
import sim.engine.Steppable;

/** totally ordered events */

public abstract class Event implements Comparable<Event>, Steppable
{
    private static int freshIndex = 0;

    protected double time;
    private int index;

    private boolean triggered;
    protected boolean cancelled;

    public Event(double delay)
        {
            index = freshIndex++;
            this.time = Engine.time() + delay;
        }

    public void setAbsTime(double abs_time)
    {
        if(abs_time > Engine.time())
            {
                this.time = abs_time;
            }
    }

    public final double time()
    {
        return time;
    }

    public final int compareTo(Event e) 
    { 
        if(time > e.time) return 1;
        else if(time < e.time) return -1;
        else if(index > e.index) return 1;
        else if(index < e.index) return -1;
        else return 0;
    }

    public final boolean triggered()
    {
        return triggered;
    }

    public final boolean pending()
    {
        return !cancelled && !triggered;
    }

    public final boolean cancelled()
    {
        return cancelled;
    }

    public final void cancel()
    {
        cancelled = true;
    }

    public final void trigger()
    {
        if(!Param.SILENT){ Engine.log.println(logRepLong() + " trigger");  }
        triggered = true;
        handle();
    }

    public abstract void handle();

    public String logRepShort()
    {
        if(Param.SILENT){ return ""; }
        return Log.getClassName(this) + "(" + time() + ")";
    }

    public String logRepLong()
    {
        if(Param.SILENT){ return ""; }
        return logRepShort();
    }

    public void step(SimState state)
    {
        if(!cancelled)
            {
                if(Param.Verbosity > 0) Engine.log.println("--- Simulation Time " + Log.tf.format(time()) + " ---");
                trigger();
            }
    }

}
