/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.common;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.ListIterator;

import cf.api.ApplicationKnowledge;

public class KnowledgeBase extends LinkedList<ApplicationKnowledge> implements Serializable
{
  public synchronized void update(ApplicationKnowledge k)
  {
    remove(k);
    add(k);
    additions = true;
  }

  public synchronized void remove(ApplicationKnowledge k)
  {
    ListIterator<ApplicationKnowledge> li = listIterator();
    while(li.hasNext())
    {
      ApplicationKnowledge e = li.next();
      // SimEngine.log.println(" - existing local knowledge: ");
      // SimEngine.log.println(" - " + e.logRepLong());

      //if(e.related(k) && k.subsumed(e) && !e.subsumed(k))
      if(e.related(k) && k.replaces(e))
      { 
        // SimEngine.log.println(" - removing local knowledge: ");
        // SimEngine.log.println(" - " + e.logRepLong());
    	  e.onReplaced();	// JS
        li.remove();
      }
    }
  }

  //public ApplicationKnowledge subsumed(ApplicationKnowledge k)
  public synchronized ApplicationKnowledge ignore(ApplicationKnowledge k)
  {
    ListIterator<ApplicationKnowledge> li = listIterator();
    while(li.hasNext())
    {
      ApplicationKnowledge e = li.next();

      //if(e.related(k) && e.subsumed(k)) 
      if(e.related(k) && (e.subsumes(k) || e.replaces(k))) 
      {
        return e;
      }
    }
    return null;
  }

//  public synchronized void remove(KnowledgeBase other_kb)
//  {
//    ListIterator<ApplicationKnowledge> li = listIterator();
//    while(li.hasNext())
//    {
//      ApplicationKnowledge e = li.next();
//
//      //if(e.related(k) && e.subsumed(k)) 
//      if( other_kb.ignore(e) != null)
//        li.remove();
//    }
//  }

  public synchronized void removeExpiredKnowledge(ApplicationKnowledge k)
  {
    //if(k.expired()) remove(k);
    ListIterator<ApplicationKnowledge> li = listIterator();
    while(li.hasNext())
    {
      ApplicationKnowledge e = li.next();
      if(e.related(k) && e.expired())
      {
        //System.out.println(e.logRepLong());
        li.remove();
      }
    }
  }

  public synchronized void removeExpiredKnowledge()
  {
    ListIterator<ApplicationKnowledge> li = listIterator();
    while(li.hasNext())
    {
      ApplicationKnowledge e = li.next();
      if(e.expired()) li.remove();
    }
  }

  public boolean additions = false;

  public void done()
  {
    additions = false;
  }

  public boolean additions()
  {
    return additions;
  }

  public boolean fresh(ApplicationKnowledge k)
  {
    if(ignore(k) == null)
      return true;
    else 
      return false;
  }

//  public synchronized boolean equal(ApplicationKnowledge k)
//  {
//    ListIterator<ApplicationKnowledge> li = listIterator();
//    while (li.hasNext())
//    {
//      ApplicationKnowledge e = li.next();
//      if (e.related(k))
//      {
//        return e.equal(k);
//      }
//    }
//    return true;
//  }


  public void print()
  {
    for(ApplicationKnowledge k : this) 
    {
      //SimEngine.rep.println(k.logRepLong());
      k.print();
    }
  }

}
