/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.common;

import java.util.LinkedList;

public class NodeList extends LinkedList<Node>
{
  public Node getNode(int index)
  {
    for (Node n : this)
    {
      if (n.index() == index) return n;
    }
    return null;
  }
  public NodeList copy()
  {
    NodeList l = new NodeList();
    for (Node n : this) l.add(n);
    return l;
  }

  public Node find(Node dest)
  {
    for (Node n : this)
    {
      if (n == dest) return n;
    }
    return null;
  }

  public String toString()
  {
    String r = null;
    for (Node pn : this)
    {
      if (r == null) r = "";
      else r+= ", ";
      r += pn.logRepShort();
    }
    return r;
  }

  public boolean sublist(NodeList l)
  {
    for (Node n : this) if (!l.contains(n)) return false;
    return true;
  }

  public boolean equal(NodeList l)
  {
    return this.sublist(l) && l.sublist(this);
  }

  public void print(boolean detailed)
  {
    for (Node pn : this)
    {
      if (detailed) pn.print();
      else Engine.rep.println("" + pn.logRepShort());
    }
  }
}