/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.common;

import java.util.LinkedList;

import cf.api.CyberApplication;
import cf.util.Dist;

abstract public class World
{
  /** CONSTRUCTOR **/
  
  //PADO
  protected long myEngineId;
  public World()
  {		
	if(Param.EngineIdSeed == 0)
		myEngineId = Engine.dist.rand().nextLong();
	else
	    myEngineId = new Dist(Param.EngineIdSeed).rand().nextLong();
	
	if(myEngineId < 0) myEngineId *= -1;
  }
  
  /* Unique id per each cyber engine in whole system */
  public long myEngineId()
  {
    return myEngineId;
  }
  //END of PADO
  
  /** MEMBER VARS **/
  
  protected boolean cyberThreadStart = false;
  protected int freshModelIndex = 0; // for unique SimModel index in the world
  protected LinkedList<Model> allModels = new LinkedList<Model>(); 
  protected LinkedList<KnowledgeManager> allKM = new LinkedList<KnowledgeManager>();
  protected NodeList allNodes = new NodeList();
  protected KnowledgeBase kb_shared = new KnowledgeBase();
  protected LinkedList<CyberApplication> allApplications = new LinkedList<CyberApplication>();

  /** MEMBER FUNCS **/
  
  abstract public void initialize();
  /* will be called when finish the world */
  abstract public void close();

  public void addModel(Model n)
  {
    if(!allModels.contains(n)) allModels.add(n);
    if(n instanceof Node) allNodes.add((Node)n);
  }

  public CyberApplication addApplication(CyberApplication app)
  {
    if(!allApplications.contains(app)) allApplications.add(app);
    return app;
  }

  public int freshModelIndex() 
  { 
    return freshModelIndex++; 
  }
  
  public LinkedList<Model> allModels() 
  { 
    return allModels; 
  }

  public LinkedList<KnowledgeManager> allKM()
  { 
    return allKM ; 
  } 

  public NodeList allNodes() 
  { 
    return allNodes; 
  }

  public KnowledgeBase kb_shared()
  { 
    return kb_shared; 
  }

  public int totModels() 
  { 
    return allModels.size(); 
  }

  public int totNodes() 
  {
    return allNodes.size(); 
  }

  /** INNER CLASSES **/

}
