/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.real;

import java.io.Serializable;
import java.net.InetAddress;

import cf.common.Param;

public class IPInfo  implements Serializable
{
  InetAddress ip;
  int port;
  String hostname;
  short prefixLength=0;
  boolean foreign;

  InetAddress ip() { return ip; }
  int port() { return port; }
  String hostname() { return hostname; }
  short prefixLength() { return prefixLength; }
  boolean isForeign() { return foreign; }
  void setForeign(boolean f) { foreign = f; }

  IPInfo(InetAddress ip, int port,  String hostname)
  {
    this.ip = ip;
    this.hostname = hostname;
    this.port = port;
  }

  IPInfo(InetAddress ip, int port, String hostname, short prefixLength)
  {
    this.ip = ip;
    this.hostname = hostname;
    this.port = port;
    this.prefixLength = prefixLength;
  }

  public String toString() 
  {
    return ip.getHostAddress() + "/" + hostname + ":" + port;
  }

  public String logRepShort()
  {
      return hostname + ":" + (port-Param.LocalPortMin);
  }

  /* redefine for "set" */
  public boolean equals(Object o)
  {
    if(o != null && getClass() == o.getClass() )
    {
      IPInfo other = (IPInfo)o;
      return ip.equals(other.ip()) && (port == other.port()); // compare IP and port
      //return hostname.equals(other.hostname()) && (port == other.port());
    }
    else
      return false;
  }

  /* redefine for "set" */
  public int hashCode()
  {
    return 31*ip.hashCode() + 17*port;
  }
}