/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.real;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;

import cf.common.Engine;
import cf.common.KnowledgeBase;
import cf.util.Log;
import cf.common.Param;

public class KnowledgeReceiver extends Thread 
{
  //DatagramSocket socket;
  DatagramSocket socket;
  RealWorld world;
  private boolean stopped = false;

  public KnowledgeReceiver(DatagramSocket socket, RealWorld world)
  {
    this.world = world;
    this.socket = socket;
  }

  String logRepShort()
  {
    return Log.getClassName(this) + "(" + socket.getLocalPort() + ")";
  }

  public void log(int level, String msg)
  {
      if(!Param.SILENT) RealEngine.log.println(level, logRepShort() + " > " + msg);
  }

  public void halt() {
    this.stopped = true;
  }

  public void run() {
    byte [] buffer = new byte[66536];
    while(true)
    {
      if(stopped)
      {
        //System.out.println("KnowledgeReceiver End");
        return;
      }

      DatagramPacket dp = new DatagramPacket(buffer, buffer.length);

      try {
        try{
          socket.receive(dp);
        }
        catch(Exception e ) { 
          //System.err.println("KnowledgeReceiver.socket"+ e);
          continue;/* do nothing*/
        }

        if( world.myIP().contains(dp.getAddress(), dp.getPort()) ) continue; // loopback ignore

        ByteArrayInputStream b_in = new ByteArrayInputStream(dp.getData());
        ObjectInputStream o_in = new ObjectInputStream(b_in);
        Object o = o_in.readObject();
        o_in.close();

        //System.out.println(logRepShort() + "Packet Received :"+ "from " + dp.getAddress().toString()+ ":"+dp.getPort() );

        if(o instanceof KnowledgeBase)
        {
          KnowledgeBase kb = (KnowledgeBase)(o);
          if(!Param.SILENT) log(2, "received knowledge via UDP from " + dp.getAddress().getHostAddress() + ":" +  dp.getPort() + " at time " + Engine.time());
          //System.out.println(logRepShort() + "Normal knowledge received from: "+ dp.getAddress() +", pid: " + kb.pid());

          //          synchronized(world.knowledgeRcvQueue()){
          //            world.knowledgeRcvQueue().add(kb);
          //          }
          //world.add_neighbor(new IPInfo(dp.getAddress(), dp.getPort(), dp.getAddress().getHostName())); // update neghibor ip if from unknown ip
          RealEngine.add(new CheckIncomingKnowledgeEvent(0, world, kb));
        }
        //IP Knowledge , Neighbor ip Information update
        else if(o instanceof IPKnowledge)
        {
          IPKnowledge ip_k = (IPKnowledge)o;
          //System.out.println("IP knowledge received: "+ip_k.toString());
          //world.handleIncomingIPKnowledge( ip_k, dp);
          RealEngine.add(new CheckIncomingIPKnowledgeEvent(0, world, ip_k, dp));
        }

      }
    catch(ClassNotFoundException e)
	{
    	Log.stderr.println("received knowledge of unknown class - " + e);
	    continue;
	}
      catch ( Exception e ) 
      {
    	Log.stderr.println(logRepShort() + "KnowledgeReceiver - " + e);
        e.printStackTrace();
        continue;
      }
    }
  }
}
