/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.real;

import cf.api.ApplicationKnowledge;
import cf.api.CyberNode;
import cf.common.KnowledgeBase;
import cf.common.Model;
import cf.common.Node;
import cf.common.Param;
import cf.common.Node.KnowledgeHandleEvent;
import cf.common.Node.KnowledgeUpdateEvent;
import cf.util.Log;

public class RealNode extends Node implements CyberNode
{
  /** CONSTRUCTOR **/
  
  public RealNode(RealWorld world)
  {
    super(world);
    this.world = world;
    km = new RealKnowledgeManager(world, this);
    super.km = this.km;
  }

  public RealNode(RealWorld world, String name)
  {
    super(world,name);
    this.world = world;
    km = new RealKnowledgeManager(world, this);
    super.km = this.km;
  }

  public RealNode(RealWorld world, Model parent)
  {
    super(world, parent);
    this.world = world;
    km = new RealKnowledgeManager(world, this);
    super.km = this.km;
  }

  public void setRealKnowledgeManager(RealKnowledgeManager km_) {
      km = km_;
      super.km=this.km;
  }

  
  public RealNode(RealWorld world, Model parent, String name)
  {
    super(world, parent, name);
    this.world = world;
    km = new RealKnowledgeManager(world, this);
    super.km = this.km;
  }

  /** MEMBER VARS **/
  
  private RealWorld world;
  private RealKnowledgeManager km;
  
  //PADO
  private boolean networkNode = false;
  
  public void setNetworkNode()
  {
    networkNode = true;
  }

  public boolean isNetworkNode()
  {
    return networkNode;
  }
  
  public void terminate()
  {
	Log.stdout.println("RealNode terminate");
    RealEngine.terminate();
    if(Param.MultiQ)
      explicit_terminated = true;
  }

//END of PADO
  
  /** MEMBER FUNCS **/
  
  public RealKnowledgeManager km() 
  { 
    return km; 
  }

  public RealNode node()
  {
    return this;
  }

  void postKnowledgeFromUDP(ApplicationKnowledge k) // request knowledge update (and publish knowledge)
  {
    log("incoming knowledge from UDP: " + k.logRepShort());

    if(Param.CyberThread)
    {
      postEvent( new KnowledgeUpdateEvent( 0, this ,k  ) );

      if(!km().kb().fresh(k) || k.expired())
        return;

      if(Param.SharedKnowledgeBase)
      {
        for(Node n : world.allNodes())
        {
          if(n != this) 
          {
            n.postEvent( new KnowledgeHandleEvent(0, n, k) ); 
          }
        }
      }
      else if(!Param.UseSimNetwork) 
      {
        for(Node n : world.allNodes())
        {
          if(n != this) 
          {
            n.postEvent( new KnowledgeUpdateEvent( 0, n ,k  ) );
          }
        }
      }
    }
    else
    {
      if(!km().handleKnowledgeUpdate(this, k))
        return;

      /* handle Knowledge on each Cyber node directly, when SharedKnowledge Base */
      if(Param.SharedKnowledgeBase)
      {
        for(Node n : world.allNodes())
        {
          if(n != this) 
          {
            n.km().manageKnowledge(k);
          }
        }
      }
      /* No link, update directly  to every node in world*/
      else if(!Param.UseSimNetwork) 
      {
        for(Node n : world.allNodes())
        {
          if(n != this) {
            n.km().handleKnowledgeUpdate(this, k);
          }
        }
      }
      //END
    }
  }

  public KnowledgeBase kb() {
	return super.km().kb();
  }
}
