/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.basic.devices;

import cf.api.CyberApplication;
import cf.api.devices.Position;
import cf.api.devices.PositionApplication;
import cf.common.Event;
import cf.common.Model;
import cf.sim.SimWorld;

public class SimPosition extends Model implements Position
{    
  private double posx, posy;

  private MoveEvent lastMoveEvent;

  public final double posx() { return posx; }
  public final double posy() { return posy; }

  public final void setPos(double posx, double posy)
  {
    this.posx = posx;
    this.posy = posy;
  }

  public SimPosition(SimWorld world)
  {
    this(world,null,null);
  }

  public SimPosition(SimWorld world, Model parent)
  {
    this(world,parent,null);
  }

  public SimPosition(SimWorld world, Model parent, String name)
  {
    super(world,parent,name);
    this.posx = 0;
    this.posy = 0;
  }

  public String logRepLong()
  {
    return super.logRepLong() + " Position: " + "(" + posx + "," + posy + ")";
  }

  public void addApplication(PositionApplication app)
  {
    addApplication((CyberApplication)app);
  }

  class MoveEvent extends Event
  {
    SimPosition position;
    double newx, newy;

    MoveEvent(SimPosition position, double delay, double newx, double newy)
    {
      super(delay);
      this.position = position;
      this.newx = newx;
      this.newy = newy;
    }

    public void handle()
    {	    
      position.handleMoveEvent(this);
    }
  }

  public void move(double newx, double newy) 	
  { 
    log("initiating move");
    if(lastMoveEvent != null && lastMoveEvent.pending()) 
    {
      lastMoveEvent.cancel();
      log("cancelling previous move");
    }
    lastMoveEvent = new MoveEvent(this,1.0,newx,newy);
    getNode().postEvent(lastMoveEvent);
  }

  private void handleMoveEvent(MoveEvent e)
  {
    log("completing move");

    posx = e.newx;
    posy = e.newy;

    for(CyberApplication app : allApplications())
    {
      if(app instanceof PositionApplication) ((PositionApplication)app).handlePositionUpdate();
    }
  }

  public void stop()
  {
    if(lastMoveEvent != null && lastMoveEvent.pending()) 
    {
      lastMoveEvent.cancel();
      log("stopping previous move");
    }
  }

}

