/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.dtn;

import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.sim.SimWorld;
import cf.util.Log;
import cf.common.Param;

public class Address
{
  private SimNode node;
  private int index;
  private String name;

  public Address(SimNode node)
  {
    this.node = node;
    this.index = node.index();
    this.name = node.name();
  }

  Address(int index, String name)
  {
    this.node = null;
    this.index = index;
    this.name = name;
  }

  public int index() 
  {
    return index;
  }

  public String name()
  {
    if (name == null) return "a" + index;
    else return name;
  }

  public String logRepShort()
  {
      if(Param.SILENT){ return ""; }
      return Log.getClassName(this) + "(" + name() + ")";
  }

  public String logRepLong()
  {
      if(Param.SILENT){ return ""; }
      return logRepShort();
  }

  public SimNode node()
  {
    return node;
  }

  public void print()
  {
      if(Param.SILENT){ return; }
      
      SimEngine.rep.println("Address " + logRepShort());
      SimEngine.rep.begin();
      SimEngine.rep.println("Bound To: " + ((node == null) ? "none" : node.logRepShort()));
      SimEngine.rep.end();
  }

  public static void printAll(SimWorld n)
  {
      if(Param.SILENT){ return; }
      
      for (Address a : n.allAddr()) {
          SimEngine.rep.println("");
          a.print();
      }
  }
    
}

