/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.dtn;

import cf.api.ApplicationKnowledge;
import cf.sim.SimEngine;
import cf.sim.SimNode;

/** Knowledge about bundles
  - in transit (used to purge buffer space off copies of this bundle which are relatively obsolete)
  - delivered at destination (used to purge buffer space off copies of this bundle) */

public class BundleDeliveredKnowledge extends ApplicationKnowledge
{
  // Uniquely determines bundle modulo copies.

  private Bundle bundle;
  private int seq;

  // Observes.

  public int seq() {
    return seq;
  }
  public Bundle bundle() {
    return bundle;
  }

  // Constructor.

  public BundleDeliveredKnowledge(SimNode creator, Bundle bundle, double creationTimee)
  {
    super(creator.name(),creator.time() , creationTimee);

    this.bundle = bundle;
    this.seq = bundle.seq();
  }

  public boolean related(ApplicationKnowledge k)
  {
    if (k instanceof BundleDeliveredKnowledge)
    {
      BundleDeliveredKnowledge bs = (BundleDeliveredKnowledge)k;
      return creator() == bs.creator() && seq == bs.seq();
    }
    else return false;
  }

  public boolean bundleDelivered(SimNode n, int seq)
  {
    //return this.creator == n && this.seq == seq;
    return this.creator().equals(n.name()) && this.seq == seq;
  }

  public String logRepShort()
  {
    return "BundleDeliveredStatus(" + creator() + ", B(" + seq + "))";
  }

  public void print()
  {
    SimEngine.rep.println(logRepShort());
    super.print();
  }

}
