/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.dtn;

import cf.common.KnowledgeBase;
import cf.common.Param;
import cf.sim.SimEngine;

/** Status updates (used by KM to disseminate knowledge about network status). */

public class KnowledgePayload extends Payload
{
  /** Sequence number of status update. */

  private int seq;

  /** Knowledge about network status. */

  //    private KnowledgeSet knowledge;
  private KnowledgeBase knowledge;

  /* Next available sequence number. */

  private static int freshSeq = 0;
  private static int freshSeq() {
    return freshSeq++;
  }

  /* Observers. */

  //public KnowledgeSet knowledge() {
  public KnowledgeBase knowledge() {
    return knowledge;
  }

  /** Class initializer. */

  public static void initialize()
  {
    freshSeq = 0;
  }

  /** Constructor. */

  //public Status(KnowledgeSet knowledge)
  public KnowledgePayload(KnowledgeBase knowledge)
  {
    this.seq = freshSeq();
    this.knowledge = knowledge;
  }

  /** Informative name. */

  public String logRepShort()
  {
    return "KnowledgePayload(" + seq + ")";
  }

  public String logRepLong()
  {
    return "KnowledgePayload(" + seq + ")";
  }

  //FIXME:cal real size byte
  public double realSize()
  {
      return Param.BundleHeaderSize + knowledge.size() * Param.AvgKnowledgeItemSize;
  }

  public void print()
  {
    SimEngine.rep.println(logRepLong());
    SimEngine.rep.begin();
    SimEngine.rep.println("Contents: ");
    knowledge.print();
    SimEngine.rep.end();
  }
}

