/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.dtn;

import cf.api.ApplicationKnowledge;
import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.util.Log;

/** knowledge about links */

public class LinkParamKnowledge extends ApplicationKnowledge
{
  /** Uniquely determines link. */

  private SimLink link;

  /** Knowledge about this link. */

  private SimNode transmitter, receiver;
  private LinkType type;
  private double latency;
  private double bandwidth;
  private double errorRate;
  private double cost;

  /** Observes. */

  public SimLink link() {
    return link;
  }

  /** Constructor. */

  public LinkParamKnowledge(SimNode creator, SimLink link)
  {
    super(creator.name(),creator.time() ,  java.lang.Double.POSITIVE_INFINITY);
    this.link = link;
    this.receiver = link.receiver();
    this.transmitter = link.transmitter();
    this.type = link.type();
    this.latency = link.latency();
    this.bandwidth = link.bandwidth();
    this.errorRate = link.errorRate();
    this.cost = link.cost();
  }

  public boolean related(ApplicationKnowledge k)
  {
    if (k instanceof LinkParamKnowledge)
    {
      LinkParamKnowledge ls = (LinkParamKnowledge)k;
      return link == ls.link;
    }
    else return false;
  }

  public void print()
  {
    SimEngine.rep.println(logRepLong());
    super.print();
    SimEngine.rep.begin();
    SimEngine.rep.println("Type: " + type);
    SimEngine.rep.println("Latency: " + Log.precision(3,latency));
    SimEngine.rep.println("Bandwidth: " + Log.precision(3,bandwidth));
    SimEngine.rep.println("ErrorRate: " + Log.precision(3,errorRate));
    SimEngine.rep.println("Cost: " + Log.precision(3,cost));
    SimEngine.rep.end();
  }

}

