/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.dtn;

import cf.api.ApplicationKnowledge;
import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.util.Log;

public class LinkStatisticsKnowledge extends ApplicationKnowledge
{
  /** Uniquely determines link. */

  private SimLink link;

  /** Knowledge about this link. */

  private double avgUpDuration;
  private double sdUpDuration;
  private double avgDownDuration;
  private double sdDownDuration;
  private double avgQueueSize;
  private double sdQueueSize;

  private double avgLongDownDuration;
  private double sdLongDownDuration;
  private double avgShortDownDuration;
  private double sdShortDownDuration;

  public double avgDownDuration()
  {
    return avgDownDuration;
  }

  public double avgUpDuration()
  {
    return avgUpDuration;
  }

  public double connectivity()
  {
    return avgUpDuration/(avgUpDuration+avgDownDuration);
  }

  /** Observes. */

  public SimLink link() {
    return link;
  }

  /** Constructor. */

  public LinkStatisticsKnowledge(SimNode creator, SimLink link)
  {
    super(creator.name(),creator.time() ,  java.lang.Double.POSITIVE_INFINITY);

    this.link = link;

    if (this.avgDownDuration == 0 && link.sensedDown()) this.avgDownDuration = SimEngine.time() - link.lastSensedDownEvent();
    if (this.avgUpDuration == 0 && link.sensedUp()) this.avgUpDuration = SimEngine.time() - link.lastSensedUpEvent();
  }

  public boolean related(ApplicationKnowledge k)
  {
    if (k instanceof LinkStatisticsKnowledge)
    {
      LinkStatisticsKnowledge ls = (LinkStatisticsKnowledge)k;
      return link == ls.link;
    }
    else return false;
  }

  public void print()
  {
    SimEngine.rep.println(logRepLong());
    super.print();
    SimEngine.rep.begin();
    SimEngine.rep.println("AvgUpDuration: " + Log.precision(3,avgUpDuration));
    SimEngine.rep.println("SDUpDuration: " + Log.precision(3,sdUpDuration));
    SimEngine.rep.println("AvgDownDuration: " + Log.precision(3,avgDownDuration));
    SimEngine.rep.println("SDDownDuration: " + Log.precision(3,sdDownDuration));
    SimEngine.rep.println("AvgShortDownDuration: " + Log.precision(3,avgShortDownDuration));
    SimEngine.rep.println("SDShortDownDuration: " + Log.precision(3,sdShortDownDuration));
    SimEngine.rep.println("AvgLongDownDuration: " + Log.precision(3,avgLongDownDuration));
    SimEngine.rep.println("SDLongDownDuration: " + Log.precision(3,sdLongDownDuration));
    SimEngine.rep.println("Connectivity: " + Log.precision(3,connectivity()));
    SimEngine.rep.end();
  }

}