/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.dtn;

import cf.api.ApplicationKnowledge;
import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.util.Log;

public class LinkStatusKnowledge extends ApplicationKnowledge
{
  /** Uniquely determines link. */

  private SimLink link;

  /** Knowledge about this link. */

  private boolean sensedUp, sensedDown;
  private double lastUpEvent, lastDownEvent;

  private int lastLongDownEventCount;
  private int lastShortDownEventCount;
  private int lastRelativeUpEventCount;
  private int minRelativeUpEventCount;
  private int maxRelativeUpEventCount;

  /** Observes. */

  public SimLink link() {
    return link;
  }

  /** Constructor. */

  public LinkStatusKnowledge(SimNode creator, SimLink link)
  {
    super(creator.name(),creator.time() ,  java.lang.Double.POSITIVE_INFINITY);
    this.link = link;
    this.sensedUp = link.sensedUp();
    this.sensedDown = link.sensedDown();
    this.lastUpEvent = link.lastSensedUpEvent();
    this.lastDownEvent = link.lastSensedDownEvent();
    this.lastLongDownEventCount = link.lastSensedLongDownEventCount();
    this.lastShortDownEventCount = link.lastSensedShortDownEventCount();
    this.lastRelativeUpEventCount = link.lastSensedRelativeUpEventCount();
    this.minRelativeUpEventCount = link.minSensedRelativeUpEventCount();
    this.maxRelativeUpEventCount = link.maxSensedRelativeUpEventCount();
  }

  public boolean related(ApplicationKnowledge k)
  {
    if (k instanceof LinkStatusKnowledge)
    {
      LinkStatusKnowledge ls = (LinkStatusKnowledge)k;
      return link == ls.link;
    }
    else return false;
  }

  public String status()
  {
    if (sensedUp) return "SensedUp";
    else if (sensedDown) return "SensedDown";
    else return "???";
  }

  public void print()
  {
    SimEngine.rep.println(logRepLong());
    super.print();
    SimEngine.rep.begin();
    SimEngine.rep.println("SensedUp: " + sensedUp);
    SimEngine.rep.println("SensedDown: " + sensedDown);
    SimEngine.rep.println("LastUpEvent: " + Log.tf.format(lastUpEvent));
    SimEngine.rep.println("LastDownEvent: " + Log.tf.format(lastDownEvent));
    SimEngine.rep.println("LastLongDownEventCount: " + lastLongDownEventCount);
    SimEngine.rep.println("LastShortDownEventCount: " + lastShortDownEventCount);
    SimEngine.rep.println("LastRelativeUpEventCount: " + lastRelativeUpEventCount);
    SimEngine.rep.println("MinRelativeUpEventCount: " + minRelativeUpEventCount);
    SimEngine.rep.println("MaxRelativeUpEventCount: " + maxRelativeUpEventCount);
    SimEngine.rep.end();
  }

}