/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.dtn;

import java.util.LinkedList;

import cf.sim.SimEngine;
import cf.sim.SimNode;

public class SimLinkList extends LinkedList<SimLink>
{
  public SimLink find(SimNode dest)
  {
    for (SimLink l : this)
    {
      if (l.receiver() == dest) return l;
    }
    return null;
  }

  public SimLinkList selectLinksTo(SimNode dest)
  {
    SimLinkList ll = new SimLinkList();
    for (SimLink l : this)
    {
      if (l.receiver() == dest) ll.add(l);
    }
    return ll;
  }

  public boolean containsLinkTo(SimNode dest)
  {
    SimLinkList ll = new SimLinkList();
    for (SimLink l : this)
    {
      if (l.receiver() == dest) return true;
    }
    return false;
  }

  public String toString()
  {
    String r = null;
    for (SimLink l : this)
    {
      if (r == null) r = "";
      else r+= ", ";
      r += l.logRepLong();
    }
    return r;
  }

  public void addToSet(SimLink l)
  {
    if (!contains(l)) add(l);
  }

  public void addAllToSet(SimLinkList ll)
  {
    for (SimLink l : ll) addToSet(l);
  }

  public void print(boolean detailed)
  {
    for (SimLink l : this)
    {
      if (detailed) l.print();
      else SimEngine.rep.println("" + l.logRepLong());
    }
  }
}