/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.dtn;

/** Transmissions are variable length units exchanged by the hardware abstraction
  layer (HAL). Currently transmission payloads can be either: knowledge bundles or data bundles
 */

public abstract class TransmissionPayload
{
  /** Informative abbreviation. */

  public abstract String logRepShort();
  public abstract String logRepLong();

  /** Size of correponding real transmission in bytes (model of messages here is an abstraction of reality). */

  public abstract double realSize();

  /** Called if transmission is disrupted due to link cut off. */

  public abstract void handleCutOff(double cutOffSize);

  /** Called if transmission is disrupted due to transmission error. */

  public abstract void handleError();

  /** Check if transmission is completely lost */

  public abstract boolean lost();

  public abstract boolean isKnowledge();
  public abstract boolean isData();
  public abstract boolean isBackground();
  public abstract boolean isApp();

  /** Print transmission. */

  public abstract void print();
}