/*

   Copyright 2010-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.mason;

import sim.portrayal.continuous.*;
import sim.engine.*;
import sim.display.*;
import javax.swing.*;

import cf.sim.SimEngine;

import java.awt.Color;

public class GUIState2DMason extends GUIState
    {
    public Display2D display;
    public JFrame displayFrame;
    public SimWorld2DMason world;
    
    ContinuousPortrayal2D coPortrayal = new ContinuousPortrayal2D();

    public GUIState2DMason(SimWorld2DMason world) { super(world.simstate); this.world = world; }

    public static String getName() { return "SimWorld2D"; }
    
    public void start()
        {
        super.start();
        setupPortrayals();
        }

    public void load(SimState state)
        {
        super.load(state);
        setupPortrayals();
        }
        
    public void setupPortrayals()
        {
    	coPortrayal.setField(world.environment);
    	display.reset();
        display.repaint();
        }

    public void init(Controller c)
        {
        super.init(c);

        display = new Display2D(600,600,this,1);
        display.setBackdrop(Color.black);
        display.attach( coPortrayal, "Agents" );

        displayFrame = display.createFrame();
        displayFrame.setTitle("SimWorld2D");
        displayFrame.setVisible(true);

        c.registerFrame(displayFrame);
        }
        
    public void quit()
        {
        super.quit();
        
        if (displayFrame!=null) displayFrame.dispose();
        displayFrame = null;
        display = null;
        SimEngine.quit();
        }

    }
