/*

   Copyright 2010-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.mason;

import sim.portrayal3d.continuous.ContinuousPortrayal3D;
import sim.portrayal3d.simple.CircledPortrayal3D;
import sim.engine.*;
import sim.display.*;
import sim.display3d.Display3D;

import javax.swing.*;
import java.awt.Color;

import cf.sim.SimEngine;

public class GUIState3DMason extends GUIState
    {
    public Display3D display;
    public JFrame displayFrame;
    public SimWorld3DMason world;
    
    ContinuousPortrayal3D coPortrayal = new ContinuousPortrayal3D();

    public GUIState3DMason(SimWorld3DMason world) {  super(world.simstate); this.world = world; }

    public static String getName() { return "SimWorld3D"; }
    
    public void start()
        {
        super.start();
        setupPortrayals();
        }

    public void load(SimState state)
        {
        super.load(state);
        setupPortrayals();
        }
        
    public void setupPortrayals()
        {
    	coPortrayal.setField(world.environment);
    	display.reset();
        display.repaint();
        }

    public void init(Controller c)
        {
        super.init(c);

        display = new Display3D(600,600,this,1);
        display.setBackdrop(Color.black);               
        display.translate(-SimWorld3DMason.XMAX/2.0,-SimWorld3DMason.YMAX/2.0,-SimWorld3DMason.ZMAX/2.0);           
        display.scale(1.0/Math.max(Math.max(SimWorld3DMason.XMAX, SimWorld3DMason.YMAX), SimWorld3DMason.ZMAX));
        display.attach( coPortrayal, "Agents" );
 
        displayFrame = display.createFrame();
        displayFrame.setTitle("SimWorld3D");
        displayFrame.setVisible(true);

        c.registerFrame(displayFrame);       
       }
        
    public void quit()
        {
        super.quit();
        
        if (displayFrame!=null) displayFrame.dispose();
        displayFrame = null;
        display = null;
        SimEngine.quit();
        }

    }
