/*

   Copyright 2010-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

 */

package cf.sim.mason.devices;

import cf.api.CyberApplication;
import cf.api.devices.Position3D;
import cf.api.devices.PositionApplication;
import cf.common.Event;
import cf.common.Model;
import cf.sim.mason.SimWorldMason;

public class SimPosition3D extends Model implements Position3D
{    
	private double posx, posy, posz;

	private MoveEvent lastMoveEvent;

	public final double posx() { return posx; }
	public final double posy() { return posy; }
	public final double posz() { return posz; }

	public final void setPos(double posx, double posy, double posz)
	{
		this.posx = posx;
		this.posy = posy;
		this.posz = posz;
	}

	public SimPosition3D(SimWorldMason world)
	{
		this(world,null,null);
	}

	public SimPosition3D(SimWorldMason world, Model parent)
	{
		this(world,parent,null);
	}

	public SimPosition3D(SimWorldMason world, Model parent, String name)
	{
		super(world,parent,name);
		this.posx = 0;
		this.posy = 0;
		this.posz = 0;
		update();
	}

	public String logRepLong()
	{
		return super.logRepLong() + " Position: " + "(" + posx + "," + posy + ")";
	}

	public void update()
	{
	}

	public void addApplication(PositionApplication app)
	{
		addApplication((CyberApplication)app);
	}

	class MoveEvent extends Event
	{
		SimPosition3D position;
		double newx, newy, newz;

		MoveEvent(SimPosition3D position, double delay, double newx, double newy, double newz)
		{
			super(delay);
			this.position = position;
			this.newx = newx;
			this.newy = newy;
			this.newz = newz;
		}

		public void handle()
		{	    
			position.handleMoveEvent(this);
		}
	}

	public void move(double newx, double newy, double newz) 	
	{ 
		log("initiating move");
		
		if(lastMoveEvent != null && lastMoveEvent.pending()) 
		{
			lastMoveEvent.cancel();
			log("cancelling previous move");
		}
		lastMoveEvent = new MoveEvent(this,1.0,newx,newy, newz);
		postEvent(lastMoveEvent);
	}

	private void handleMoveEvent(MoveEvent e)
	{
		log("completing move");

		posx = e.newx;
		posy = e.newy;
		posz = e.newz;
		update();

		for(CyberApplication app : allApplications())
		{
			if(app instanceof PositionApplication) ((PositionApplication)app).handlePositionUpdate();
		}
	}

}

