/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.pn;

import cf.sim.SimEngine;
import cf.sim.dtn.SimLink;

/* Contact: a link between places that should open up according
to the resource schedule, i.e. when places are suitably marked. */

class Contact
{
  private SimLink link;
  private Place transmitter;
  private Place receiver;

  public Contact(SimLink link, Place transmitter, Place receiver)
  {
    this.link = link;
    this.transmitter = transmitter;
    this.receiver = receiver;
  }

  public boolean open(SimLink l)
  {
    return link == l && transmitter.contains(l.transmitter()) && receiver.contains(l.receiver());
  }

  public void print()
  {
    SimEngine.rep.println("Places: " + transmitter.logRepShort() + " => " + receiver.logRepShort());
    SimEngine.rep.println("  Associated Link: " + link.logRepShort());
  }
}
