/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.pn;

import cf.common.Node;
import cf.sim.SimEngine;
import cf.sim.SimNode;

/* Transitions that can move tokens representing a particular node. */

public class Transition
{
  private int index;
  private String name;

  private PlaceList preset, postset;

  public PlaceList preset() {
    return preset;
  }
  public PlaceList postset() {
    return postset;
  }

  private SimMobility sm;

  public Transition(SimMobility sm, int index, PlaceList preset, String name, PlaceList postset)
  {
    this.sm = sm;
    this.index = index;
    this.name = name;
    this.preset = preset;
    this.postset = postset;
  }

  /* Is transition enabled for given binding ? */

  public boolean enabled(SimNode n)
  {
    for (Place p : preset)
    {
      if (!p.available(n)) return false;
    }
    return true;
  }

  /* Is transition enabled for some binding ? */

  public SimNode enabled()
  {
    for (Node n : sm.world.allNodes())
    {
      if (enabled((SimNode)n)) return (SimNode)n;
    }
    return null;
  }

  /* The following methods require commit at higher level. */

  /* Activate transition, i.e. comsume tokens. */

  public boolean activate(SimNode n)
  {
    for (Place p : preset)
    {
      if (!p.remove(n)) return false;
    }
    return true;
  }

  /* Fire transition, i.e. comsume and produce tokens. */

  public void fire(SimNode n)
  {
    for (Place p : preset)
    {
      if (!p.remove(n)) assert false;
    }
    for (Place p : postset)
    {
      p.add(n);
    }
  }

  public String name()
  {
    if (name == null) return "" + index;
    else return name;
  }

  public String logRepShort()
  {
    return "T(" + name() + ")";
  }

  public String logRepLong()
  {
    return logRepShort();
  }

  public void print()
  {
    SimEngine.rep.println("Transition " + logRepShort());
    SimEngine.rep.begin();
    SimEngine.rep.println("Preset: " + preset.toString());
    SimEngine.rep.println("Postset: " + postset.toString());
    SimEngine.rep.end();
  }

}
