/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.stage.devices;

import cf.api.CyberApplication;
import cf.api.devices.Laser;
import cf.api.devices.LaserApplication;
import cf.api.devices.LaserSample;
import cf.common.Model;
import cf.sim.SimWorld;
import cf.sim.stage.JavaStageCtrlCyberFramework;
import cf.sim.stage.LaserInterface;
import cf.sim.stage.LaserSampleStage;

public class SimLaserStage extends Model implements Laser
{
  public JavaStageCtrlCyberFramework ctrl;
  private LaserSample [] samples;
  private int count;

  public SimLaserStage(JavaStageCtrlCyberFramework ctrl, SimWorld world)
  {
    super(world);
    this.ctrl = ctrl;
  }
  public SimLaserStage(JavaStageCtrlCyberFramework ctrl, SimWorld world, String name)
  {
    super(world, name);
    this.ctrl = ctrl;
  }
  public SimLaserStage(JavaStageCtrlCyberFramework ctrl, SimWorld world, Model parent , String name)
  {
    super(world, parent, name);
    this.ctrl = ctrl;
  }

  public void addApplication(LaserApplication app)
  {
    addApplication((CyberApplication)app);
  }

  public LaserSample[] getSamples()
  {
    LaserSampleStage [] ls = LaserInterface._GetSamples(ctrl.GetRobotP());
    if(ls !=null)
    {
      count = ls.length;
      LaserSample [] t_ls = new LaserSample[ls.length];
      for(int i=0;i< ls.length ;i ++) 
      {
        t_ls[i] = new LaserSample( ls[i].range, ls[i].reflectance );
      }
      return t_ls;
    }
    return  null;
  }

  public String logRepLong() 
  {
    return super.logRepLong() + "LaserSample Count[" + count+ "]";
  }

  public void initialize()
  {
    getNode().postEvent(1.0);
  }

  protected void handleEvent()
  {
    //log("laser info update");
    for(CyberApplication app : allApplications())
    {
      if(app instanceof LaserApplication) ((LaserApplication)app).handleLaserUpdate();
    }
    getNode().postEvent(1.0);
  }
}
