/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.stage.devices;

import cf.api.CyberApplication;
import cf.api.devices.Powerpack;
import cf.api.devices.PowerpackApplication;
import cf.common.Model;
import cf.sim.SimWorld;
import cf.sim.stage.JavaStageCtrlCyberFramework;
import cf.sim.stage.PowerPackInterface;

public class SimPowerpackStage extends Model implements Powerpack
{
  public JavaStageCtrlCyberFramework ctrl;
  private double stored, dissipated;

  public SimPowerpackStage(JavaStageCtrlCyberFramework ctrl, SimWorld world)
  {
    super(world);
    this.ctrl = ctrl;
  }
  public SimPowerpackStage(JavaStageCtrlCyberFramework ctrl, SimWorld world, String name)
  {
    super(world, name);
    this.ctrl = ctrl;
  }
  public SimPowerpackStage(JavaStageCtrlCyberFramework ctrl, SimWorld world, Model parent, String name)
  {
    super(world, parent, name);
    this.ctrl = ctrl;
  }

  public void addApplication(PowerpackApplication app)
  {
    addApplication((CyberApplication)app);
  }

  public String logRepLong() 
  {
    return super.logRepLong() + " Powerpack- stored: " + stored + " , -dissipated: " + dissipated+ " .";
  }
  
  public double GetStored() 
  {
	stored = cf.sim.stage.PowerPackInterface._GetStored(ctrl.GetRobotP());
    return stored;
  }

  public double GetDissipated()
  {
    dissipated = cf.sim.stage.PowerPackInterface._GetDissipated(ctrl.GetRobotP());
    return dissipated;
  }

}
