/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.util;

import java.util.Random;

/** Random numbers and distributions. */

public class Dist 
{    
  private Random rand;

  public static Dist dist = new Dist(System.currentTimeMillis());

  public Dist(long seed)
  {
    rand = new Random(seed);
  }

  public int sampleUniform(int l, int u)
  {
    return l + rand.nextInt(u - l);
  }

  public double sampleUniform(double l, double u)
  {
    return l + rand.nextDouble() * (u - l);
  }

  public boolean sampleBerWithP(double p)
  {
    return (rand.nextDouble() < p); 
  }

  public double sampleExpWithMean(double m)
  {
    return m * (- Math.log(rand.nextDouble()));
  }

  public double sampleExpWithRate(double r)
  {
    return (- Math.log(rand.nextDouble())) / r;
  }

  public Random rand() 
  {
    return rand;
  }
}
