/*

  Copyright 2010-2013 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.util;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DecimalFormat;

import cf.common.Param;

/** logging on console and in files */

public class Log  {
    
    public static int Verbosity = Param.Verbosity; // verbosity of logging 0-3
    
    public static PrintWriter stdout = new PrintWriter(System.out, true);
    public static PrintWriter stderr = new PrintWriter(System.err, true);
    
    private File logfile = null;
    private PrintWriter log = stdout;
    
    private int indent = 0;
    
    //private static int vstack[] = new int[10];
    //private static int vsp = 0;
    
    public static final DecimalFormat tf = new DecimalFormat("0000.000");
    
    public static final DecimalFormat df[] = new DecimalFormat[7];
    static 
    {
        df[0] = new DecimalFormat("0");
        df[1] = new DecimalFormat("0.0");
        df[2] = new DecimalFormat("0.00");
        df[3] = new DecimalFormat("0.000");
        df[4] = new DecimalFormat("0.0000");
        df[5] = new DecimalFormat("0.00000");
        df[6] = new DecimalFormat("0.000000");
    }
    
    public Log(String fn) throws IOException
    {
        if(Param.SILENT){  return; }
        
        logfile = new File(fn);
        
        log = new PrintWriter(new FileWriter(logfile), true); // autoflush       
    }
    
    public void close() 
    {
	if (log != null)
            log.close();
        log = null;
    }
    
    public void begin()
    {
        indent+=2;
    }
    
    /*private static void beginVerbose(int verbosity)
    {
        vstack[vsp++] = Verbosity;
        Verbosity = verbosity;
    }
    
    private static void endVerbose()
    {
        Verbosity = vstack[--vsp];
    }*/
    
    private void println(PrintWriter out, String s)
    {
        for(int i = 0; i < indent; i++) out.print(" ");
        out.println(s);
        out.println();
    }
    
    public void println(String s)
    {
        if(Param.SILENT){  return; }
        println(log,s);
    }
    
    public void end()
    {
        indent-=2;
        if(indent < 0) indent = 0;
    }
    
    public void println(int verbosity, String s)
    {
        if(Param.SILENT){ return; }
        if(verbosity <= Verbosity) println(log,s);
        if(verbosity == 0) Log.stdout.println(s);
    }
    
    public static String precision(int n, double x)
    {
        return Log.df[n].format(x);
    }
    
    public static String fraction(int prec, double frac, double total)
    {
        if(total == 0) return "?";
        else return Log.precision(prec,frac/total);
    }
    
    public static String percentage(double frac, double total)
    {
        if(total == 0) return "?%";
        else return Log.precision(2,100.0*frac/total) + "%";
    }
    
    public static String getClassName(Object o){
        /* In silent mode; leave the class name intact */
        if(Param.SILENT){ 
            return o.getClass().getName(); 
        } else {
            String c = o.getClass().toString();
            int i = c.lastIndexOf('.');
            if(i >= 0) return c.substring(i+1);
            int j = c.lastIndexOf(' ');
            if(j >= 0) return c.substring(j+1);
            else return c;
        }
    }
}
